"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.match_ascending_triangle = void 0;
const feature_statistics_1 = require("../feature_statistics");
const feature_regression_1 = require("../feature_regression");
const feature_optimize_1 = require("../feature_optimize");
const match_ascending_triangle = (sampleOHLC, patternOHLC) => {
    const sampleHigh = (0, feature_statistics_1.stats_normalize)(sampleOHLC.map((candle) => candle.high));
    const sampleLow = (0, feature_statistics_1.stats_normalize)(sampleOHLC.map((candle) => candle.low));
    const patternHigh = (0, feature_statistics_1.stats_normalize)(patternOHLC.map((candle) => candle.high));
    const patternLow = (0, feature_statistics_1.stats_normalize)(patternOHLC.map((candle) => candle.low));
    const objective = (scale) => {
        const resampledPatternHigh = (0, feature_statistics_1.stats_resample_data)(patternHigh, Math.round(sampleHigh.length * scale));
        const resampledPatternLow = (0, feature_statistics_1.stats_resample_data)(patternLow, Math.round(sampleLow.length * scale));
        // Pad or truncate the resampled pattern to match the sample length
        const paddedPatternHigh = resampledPatternHigh.length < sampleHigh.length ? [...resampledPatternHigh, ...new Array(sampleHigh.length - resampledPatternHigh.length).fill(resampledPatternHigh[resampledPatternHigh.length - 1])] : resampledPatternHigh.slice(0, sampleHigh.length);
        const paddedPatternLow = resampledPatternLow.length < sampleLow.length ? [...resampledPatternLow, ...new Array(sampleLow.length - resampledPatternLow.length).fill(resampledPatternLow[resampledPatternLow.length - 1])] : resampledPatternLow.slice(0, sampleLow.length);
        const similarityHigh = (0, feature_statistics_1.stats_calculate_similarity)(sampleHigh, paddedPatternHigh);
        const similarityLow = (0, feature_statistics_1.stats_calculate_similarity)(sampleLow, paddedPatternLow);
        const lowTrend = (0, feature_regression_1.linear_regression)(sampleLow).slope;
        const highTrend = (0, feature_regression_1.linear_regression)(sampleHigh).slope;
        const triangleScore = lowTrend - Math.abs(highTrend);
        // Normalize each component
        const normalizedSimilarityHigh = similarityHigh / (similarityHigh + 1);
        const normalizedSimilarityLow = similarityLow / (similarityLow + 1);
        const normalizedTriangleScore = triangleScore / (Math.abs(triangleScore) + 1);
        // Add a penalty for scaling away from 1
        const scalePenalty = Math.abs(Math.log(scale));
        return -(normalizedSimilarityHigh + normalizedSimilarityLow + normalizedTriangleScore - scalePenalty);
    };
    // Optimize the scaling factor
    const result = (0, feature_optimize_1.optimize)(objective, { bounds: [0.1, 2] });
    const bestScale = result.x;
    const bestScore = -result.fx;
    return [bestScore, bestScale];
};
exports.match_ascending_triangle = match_ascending_triangle;
//# sourceMappingURL=match_ascending_triangle.js.map